# Ilustration on how to make some presentins formats with imputed data
# Removendo todos os objetos da rea de trabalho
rm(list = ls())
while(search()[2] != "tools:rstudio") detach(2)

# install.packages("DynNom")
# install.packages("party")
library("rms")

# Load workspace with imputed data
# Dados e objetos salvos da sesso de imputao
load("C:/Banco/curso_CPM/2019/data/TBI1.RData");ls()

# save datadistributions at options workspace as many rms functions damands it
# TBI1.c1 is de TBI1 with all imputed values
dd <- datadist(TBI1.c1)
options(datadist = NULL)
options(datadist = "dd")

# Fit of any model for unfavorable outcome after trauma prediction
f1 <- lrm(unfav ~ age + motor + pupil + hypoxia + hypotens + ctclass + tsah, data = TBI1.c1, x = T, y = T)
f1
plot(summary(f1, vnames = "labels"), xlim = c(0,25))

# Making a score chart from the regression coeficients
# Assigning scores is rather arbitrary - but it is good practice to make them from the regression coeficients
# Tips:
# (1) round only in the final step
# (2) Think about reasonable limits of the total score
# (3) Think about adding a constant to turn negative coeficients into positive ones

# here age has zero points in rounding
f1$coefficients*10
# later the rounding will work better in age if 20 is used
f1$coefficients*20

# Making a data frame with the predictions and the individual and total scores
# Turning the linear predictor into probabilities
pred <- data.frame(predictions = plogis(predict(f1)))
head(pred)
tail(pred)

# Assingning the score to each patient with the appropriate characteristic
pred$age <- TBI1.c1$age * f1$coefficients["age"]
head(pred)
tail(pred)
pred$motor <- NA
pred$motor[which(TBI1.c1$motor=="1")] <- 0
pred$motor[which(TBI1.c1$motor=="2")] <- f1$coefficients["motor=2"]
pred$motor[which(TBI1.c1$motor=="3")] <- f1$coefficients["motor=3"]
pred$motor[which(TBI1.c1$motor=="4")] <- f1$coefficients["motor=4"]
pred$motor[which(TBI1.c1$motor=="5/6")] <- f1$coefficients["motor=5/6"]
head(pred)
tail(pred)
pred$pupil <- NA
pred$pupil[which(TBI1.c1$pupil=="both reactive")] <- 0
pred$pupil[which(TBI1.c1$pupil=="one reactive")] <- f1$coefficients["pupil=one reactive"]
pred$pupil[which(TBI1.c1$pupil=="no reactive pupils")] <- f1$coefficients["pupil=no reactive pupils"]
head(pred)
tail(pred)
pred$hypoxia <- ifelse(TBI1.c1$hypoxia=="Yes",f1$coefficients["hypoxia=Yes"],0)
head(pred)
tail(pred)
pred$hypotens <- ifelse(TBI1.c1$hypotens=="Yes",f1$coefficients["hypotens=Yes"],0)
head(pred)
tail(pred)
pred$ctclass <- NA
pred$ctclass[which(TBI1.c1$ctclass=="1 or 2")] <- 0
pred$ctclass[which(TBI1.c1$ctclass=="3 or 4")] <- f1$coefficients[11]
pred$ctclass[which(TBI1.c1$ctclass=="5 or 6")] <- f1$coefficients[12]
head(pred)
tail(pred)
pred$tsah <- ifelse(TBI1.c1$tsah=="Yes",f1$coefficients[13],0)
head(pred)
tail(pred)

# This step is optional and irrelevant
# Adding a constant to all scores to make them all positive
# The constant here will make the lowest coefficient = 0
min(f1$coef)
pred[,2:ncol(pred)] <- pred[,2:ncol(pred)] + abs(min(f1$coefficients))
tail(pred)

# Rounding the predictors scores
pred[,2:ncol(pred)] <- round(pred[,2:ncol(pred)] * 20)
head(pred)

# Making a total score by summing the individual score
pred$total.score <- rowSums(pred[,2:ncol(pred)])
head(pred)
tail(pred)

# Making a table just for the plot - here we want all point less the replicated for more smooth curve
pred2 <- pred[order(pred$total.score,pred$predictions),c("predictions","total.score")]
pred2[which.max(pred2$predictions),]
pred2[which.max(pred2$total.score),]
tail(pred2)
head(pred2)
table(duplicated(pred$predictions))
table(duplicated(pred$total.score))
table(duplicated(pred$total.score,pred$predictions))
# Due to rounding, there are more total.scores duplicated then the predicted risks
# It means that the same total score is assigned to different predicted risks
# Approximation is required

# Making an aditional dataset with rounded predictions and aproximated total.scores
pred3 <- data.frame(predictions = unique(round(pred2$predictions,2)))
tail(pred3)
# Assigning for each predicted risk, the related mean total score
pred3$total.score <- round(sapply(pred3$predictions,function(i)max(pred2$total.score[which(round(pred2$predictions,2)==i)])))
# This plot still not good as some "steps" are visible, which means some duplicated total.scores exists.
plot(pred3[,2:1],type="l",lwd=3)
# But this table will be nice for printing, lets keep the unique combinations

# Making the predictors individual score
# A good result would be the score table + (total score chart or total score table)
# Making the score table from the coefficients
coef.scores <- data.frame(Variable=names(f1$coefficients)[-1],Score=unname(round((f1$coefficients[-1] + abs(min(f1$coefficients)))* 20)))
coef.scores

# The table score is not complete as the reference categoreis are not shown
# Age is a tricky score, as it is a continuous variable
# Making a adataframe with each score for each value of age in the dataframe
# If age has a non-linear effect, this operation should occur for each range between the knots
range(TBI1.c1$age)
temp <- unique(data.frame(age=seq(15,80,1),age.score=unname(round(((f1$coef["age"] * seq(15,80,1)) + abs(min(f1$coef)))* 20))))
temp

# Arbitraly selecting some values of age to show in score table
temp <- temp[which(temp$age  %in% seq(15,80,5)),]
temp$age <- paste0("age = ",temp$age)

# Binding age to the table score
names(temp) <- names(coef.scores)

# As the negatives score are beeing removed, all reference values of the predictors must receive the same constant
const <- round(abs(min(f1$coef))* 20)
coef.scores <- rbind(temp,c("motor=1",const),coef.scores[-1,])
rownames(coef.scores) <- 1:nrow(coef.scores)
coef.scores <- rbind(coef.scores[1:19,],c("pupil=both reactive",const),coef.scores[20:21,],c("hypoxia=No",const),coef.scores[22,],c("hypotens=No",const),coef.scores[23,],c("ctclass=1 or 2",const),coef.scores[24:25,],c("tsah=No",const),coef.scores[26,])
coef.scores
# coef.scores[1:14,1] <- paste0("age = ",coef.scores[1:14,1])
coef.scores
rm(temp)

# Back to the pred2 dataset to make the score chart, lets remove the duplicated values and not round the predicted risk to make a smooth plot
pred2 <- pred2[order(pred2$predictions),c("total.score","predictions")] ; nrow(pred2)
pred2 <- unique(pred2) ; nrow(pred2)

table(duplicated(pred2$predictions))
table(duplicated(pred2$total.score))
pred2 <- pred2[-which(duplicated(pred2$total.score)),] ; nrow(pred2)

plot(pred2[,2:1],type="n",lwd=4) # plot without a line
axis(2,seq(0,350,5),label=F,tcl=-.1)
axis(1,seq(0,1,.01),label=F,tcl=-.1)
axis(1,seq(0,1,.05),label=F,tcl=-.3)
abline(h=seq(0,350,5),lty=3,col=gray(c(.8,rep(.95,3))))
abline(v=seq(0,1,.05),lty=3,col=gray(c(.8,rep(.95,3))))
lines(pred2[,2:1],lwd=4)

# All this is way more simple usng rms::nomogram function
# Making a nomogram
# Calling the nomogram function with additional arguments
nomo.f1 <- nomogram(f1,fun=plogis, fun.at=c(.001,.01,.05,seq(.1,.9,by=.1),.95,.99,.999),funlabel="Risk of unfavarable \n outcome",lp=F,conf.lp="all",conf.int = F)
# Everything we did by assigning scores the nomogram function does in one hit, although it assigned different scores
nomo.f1

# removing excess of margins to make a plot
par(mar=c(1,1,1,1))
# Calling the nomogram plot with additional arguments
plot(nomo.f1,col.grid=gray(c(0.8, 0.95)), cex.var = .9, cex.axis = .7)
dev.off()

# Marking a score chart from the nomogram object
# Making a table from the nomogram, this table contains the total scores and the related predicted risk
nomo.f2 <- nomogram(f1,fun=plogis, fun.at = seq(0,1,by=.01), funlabel="Risk of unfavarable \n outcome",conf.lp="all")
score.table <- data.frame(Score = nomo.f2$"Risk of unfavarable \n outcome"$x, Risk = as.numeric(nomo.f2$"Risk of unfavarable \n outcome"$x.real))
names(score.table) <- c("total score","predictions")
score.table

# Calling the plot from the table
# Orignal margins
par(mar=c(5.1,4.1,4.1,2.1))
plot(score.table[,2:1],type = "n",
     ylab = "Total score", xlab = "Predicted probability")
axis(2,seq(0,300,10),label=F,tcl=-.1)
axis(1,seq(0,1,.01),label=F,tcl=-.1)
axis(1,seq(0,1,.05),label=F,tcl=-.3)
abline(h=seq(0,300,10),lty=3,col=gray(c(.8,rep(.95,4))))
abline(v=seq(0,1,.05),lty=3,col=gray(c(.8,rep(.95,3))))
lines(score.table[,2:1],type='l',lwd=3)

# Using regression trees
# the most simple way, but there is a lot of aditional arguments that may be more appropriate
# install.packages("party")
library("party")
tree <- ctree(unfav ~  age + motor + pupil + hypoxia + hypotens + ctclass + tsah, data = TBI1.c1)

x11(width = 480*2, height = 480) ; plot(tree,type="simple") # terminal_panel = node_barplot
# y = the fraction of outcomes
dev.off()

# Shiny app allows make calclulator from the model to show it online with R code
# https://shiny.ini.fiocruz.br/pedrobrasil
# https://shiny.rstudio.com/

# Fazendo um app shiny mais automatizado
# install.packages(Dymnom)
library(DynNom)
DynNom(f1, data = TBI1.c1, covariate = "numeric")

f2 <- glm(unfav ~ age + motor + pupil + hypoxia + hypotens + ctclass + tsah, data = TBI1.c1, x = T, y = T, family = "binomial")
f2

# DNBuilder creates a folder in the working directory with the ui.R and server.R files to deploy the app
setwd("C:/Banco/curso_CPM/2019/")
DNbuilder(f1, data = TBI1.c1, covariate = "numeric")
# ou
DNbuilder(f2, data = TBI1.c1, covariate = "numeric")

# while(search()[2] != "tools:rstudio") detach(2)
# save.image("C:/Banco/curso_CPM/2019/data/TBI1_presentation.RData")
