# IMPACT illustrations of dealing with missing values
# Here, the examples are only for multiple imputation

# There are several packages with functions to conduct data imputation
# THe first example is with the rms package, the second is with mice package

# Detach packages, remove objects and clear graphical devices.
while(search()[2] != "tools:rstudio") detach(2)
rm( list = ls())
graphics.off()


# First using the rms library
# install.packages("rms")
library("rms")

# Importing the TBI dataset
# TBI dataset is an example dataset from Styerberg's book "Clnical prediction models"
TBI <- spss.get('D:/Banco/curso_CPM/2020/data/TBI.sav')

# Printing the dataset description
# page(describe(TBI),'print')

# Alternative way to show the dataset description
# It saves the description on the disk, then it shows in the notepad
sink('D:/Banco/curso_CPM/2020/data/TBI_description.txt')
describe(TBI)
sink();file.show('D:/Banco/curso_CPM/2020/data/TBI_description.txt')

# Passando as etiquetas das colunas (Hmisc::label) para atributo do banco para funo da tabela reconhecer as etiquetas
attr(TBI, "var.labels") <- label(TBI)
# install.packages("ems")
ems::tableStack(c(cause,age,d.motor,d.pupil,pupil.i,hypoxia,hypotens,ctclass,tsah,edh,shift,d.sysbpt,glucose,glucoset,ph,sodium,sodiumt,hb,hbt),by = d.mort ,dataFrame = TBI, test = FALSE, total.column = TRUE, NArow = TRUE)

# Some edition required
# Motor in 5 categories: 1/2; 3; 4; 5/6; 9/NA
table(TBI$d.motor)
TBI$d.motor	<- as.factor(TBI$d.motor)
levels(TBI$d.motor)[5:6]	<- "5/6"

# Editing mortality and unfavorables outcomes
table(TBI$d.gos) # GOS is Glasgow outcome scale - outcome at six months after hospital admission

TBI$mort <- ifelse(TBI$d.gos=="dead",1,ifelse(is.na(TBI$d.gos),NA,0))
label(TBI$mort) <- "Death at six months"
table(TBI$mort)

TBI$deadveg <- ifelse(TBI$d.gos=="dead"|TBI$d.gos=="vegetative",1,ifelse(is.na(TBI$d.gos),NA,0))
label(TBI$deadveg) <- "Dead or vegetative at six months"
table(TBI$deadveg)

TBI$unfav <- ifelse(TBI$d.gos=="dead"|TBI$d.gos=="vegetative"|TBI$d.gos=="severe disability"|TBI$d.gos=="moderate disability",1,ifelse(is.na(TBI$d.gos),NA,0))
label(TBI$unfav) <- "Unfavorable outcomes at six months"
table(TBI$unfav)

TBI$good  <- ifelse(TBI$d.gos=="good recovery",1,ifelse(is.na(TBI$d.gos),NA,0))
label(TBI$good) <- "Good recovery at six months"
table(TBI$good)

# Making dummy variables accordin to CT scan classification
TBI$ctclass2  <- ifelse(TBI$ctclass==2,1,0)
label(TBI$ctclass2) <- "Marshall CT class 2"
table(TBI$ctclass2)

TBI$ctclass1  <- ifelse(TBI$ctclass==1,1,0)
label(TBI$ctclass1) <- "Marshall CT class 1"
table(TBI$ctclass1)

TBI$ctclass34 <- ifelse(TBI$ctclass==3 | TBI$ctclass==4,1,0)
label(TBI$ctclass34) <- "Marshall CT class 3 or 4"
table(TBI$ctclass34)

TBI$ctclass56 <- ifelse(TBI$ctclass==5 | TBI$ctclass==6,1,0)
label(TBI$ctclass56) <- "Marshall CT class 5 or 6"
table(TBI$ctclass56)

# Recoding the CT classification grouping the last four categories
TBI$ctclassr4 <- ifelse(TBI$ctclass1==1,1,
                        ifelse(TBI$ctclass2==1,2,
                               ifelse(TBI$ctclass34==1,3,
                                      ifelse(TBI$ctclass56==1,4,NA))))
label(TBI$ctclassr4) <- "Marshall CT class"
TBI$ctclassr4 <- as.factor(TBI$ctclassr4)
levels(TBI$ctclassr4) <- c("1","2","3 or 4","5 or 6")
table(TBI$ctclassr4)

# Recoding the CT classification grouping into 3 classes
TBI$ctclassr3   <- ifelse(TBI$ctclass1==1 | TBI$ctclass2==1,1,
                          ifelse(TBI$ctclass34==1,2,
                                 ifelse(TBI$ctclass56==1,3,NA)))
TBI$ctclassr3	<- as.factor(TBI$ctclassr3)
label(TBI$ctclassr3) <- "Marshall CT class"
levels(TBI$ctclassr3) <- c("1 or 2","3 or 4","5 or 6")
table(TBI$ctclassr3)

## Subset TBI1 for imputation, use later to define R2 R3 and R4
## Watch out that editions will be done in a different dataset from now on
## TBI1 will be actually used for mssing diagnosis, imputation and analysis
TBI1 <- TBI[,Cs(trial, age, hypoxia, hypotens, cisterns, shift,tsah, edh, d.pupil, d.motor, ctclassr3,d.sysbpt,hbt,glucoset, unfav, mort)]
dim(TBI1);dim(TBI)

# Turning the trial number into categories
# TBI1$trial <- as.factor(TBI1$trial)
# label(TBI1$trial) <- "Trial number"

# Changing some of the TBI1 variables names
names(TBI1) <- Cs(trial, age, hypoxia, hypotens, cisterns, shift,
                  tsah, edh, pupil, motor, ctclass,
                  d.sysbp,hb,glucose, unfav, mort)

# Dichotomizing age at 30
TBI1$Age30  <- ifelse(TBI1$age < 30, 1, 0)
label(TBI1$Age30) <- "29 or younger"

# Dichotomizing cistern compression
TBI1$cisterns <- ifelse(TBI1$cisterns > 1, 1, 0)
label(TBI1$cisterns) <- "At least one cistern compressed"
table(TBI1$cisterns)

TBI1$hypoxia <- factor(TBI1$hypoxia,labels = c("No","Yes"))
label(TBI1$hypoxia) <- "Hypoxia before / at admission"
table(TBI1$hypoxia)

TBI1$hypotens <- factor(TBI1$hypotens,labels = c("No","Yes"))
label(TBI1$hypotens) <- "Hypotension before / at admission"
table(TBI1$hypotens)

TBI1$cisterns <- factor(TBI1$cisterns,labels = c("No","Yes"))
label(TBI1$cisterns) <- "At least one cistern compressed"
table(TBI1$cisterns)

TBI1$shift <- factor(TBI1$shift,labels = c("No","Yes"))
label(TBI1$shift) <- "Midline shift > 5 mm at CT"
table(TBI1$shift)

TBI1$tsah <- factor(TBI1$tsah,labels = c("No","Yes"))
label(TBI1$tsah) <- "tSAH at CT"
table(TBI1$tsah)

TBI1$edh <- factor(TBI1$edh,labels = c("No","Yes"))
label(TBI1$edh) <- "EDH at CT"
table(TBI1$edh)

# TBI1$pupil<-as.factor(TBI1$pupil)
attr(TBI1, "var.labels") <- label(TBI1)
ems::tableStack(c(trial,age,hypoxia,hypotens,cisterns,shift,tsah,edh,pupil,motor,ctclass,d.sysbp,hb,glucose,Age30),by = mort ,dataFrame = TBI1, test = FALSE, total.column = TRUE, NArow = TRUE)

# Storing data distributions in options enviroment is required for several rms functions to work
dd <- datadist(TBI1)
options(datadist = NULL) # removes previos stored datadist - optional
options(datadist = "dd")

# Show data discription
# page(describe(TBI1),'print')

# Alternative way to show the dataset description
# It saves the description on the disk, then it shows in the notepad
sink('D:/Banco/curso_CPM/2020/data/TBI1_description.txt')
describe(TBI1)
sink();file.show('D:/Banco/curso_CPM/2020/data/TBI1_description.txt')

##########################
## Missing value analysis
# par(mfrow=c(1,1)) # Optional - need only if number of windows not properly set

na.patterns <- naclus(TBI1)
# Make the cluster plot without and with labels and check if labels are in correct place
plot(na.patterns, ylab="Fraction of NAs in common")
plot(na.patterns, ylab="Fraction of NAs in common",labels = label(TBI1))

par(mfrow=c(2,2)) # The deafult option is to plot all four plots. One must set the window to seem them all at the same time
naplot(na.patterns)
dev.off() # closes the device, therefore sets the graphical paramenter back to default values

# One at the time
naplot(na.patterns,"na per var")
naplot(na.patterns,"na per obs")

na.pattern(TBI1) # hard to understand, takes a while and not very usefull

# Using the aregImpute for multiple imputation
# Watch out that aregImpute automatically transforms continuous variables with rcs function.
# This is usually good but may complicate stuff, specially in small datasets
# The function I() forces the the variable not to be transformed, good for 0 1 variables that represent categories and to force linearity for continuous variables
# It may take a while
rms.imp <- aregImpute(~ hypoxia + hypotens + cisterns + shift + tsah + edh + pupil + ctclass + d.sysbp + hb + glucose + I(unfav) , data = TBI1, n.impute = 10, tlinear = FALSE)
# will return the imputed values in all variables from all created datasets
rms.imp$imputed
print(rms.imp)

# Run some diagnostics of the imputation, not very friendly although
# In the example, will create 10 plots, one for each variable, set window to show them all
x11(width = 480*2, height = 480)
par(mfrow=c(2,5))
plot(rms.imp)
dev.off()
# Slightly different with histograms
x11(width = 480*2, height = 480)
par(mfrow=c(2,5))
plot(rms.imp,type='hist')
dev.off()

# Plot boxplots of the original values in dataset beside the imputed values
# 1 is the original data, 2:11 are the imputed values
boxplot(c(list(TBI1$hb),lapply(1:10,function(i)c(rms.imp$imputed$hb[,i]))),ylab="Hb")
boxplot(c(list(TBI1$glucose),lapply(1:10,function(i)c(rms.imp$imputed$glucose[,i]))),ylab="Glucose")

# Creating a complete dataset with the first imputed values
# Creating a clone dataset to be completed
TBI1.c1 <- TBI1

# Extract the complete varibles from the imputation object
TBI1.imputed <- impute.transcan(rms.imp, imputation = 1, data=TBI1, list.out=TRUE, pr=FALSE, check=FALSE)
# Pass these variables to the clone dataset to be completed
TBI1.c1[names(TBI1.imputed)] <- TBI1.imputed
# The original has missings, the copleted has not
table(is.na(TBI1))
table(is.na(TBI1.c1))

# Run a logistic regression analysis will 10 imputed dataset simultaneously
# It returns the rate of missing information and variance inflation for each variable
fit.mult.impute(unfav ~ trial + age + motor + pupil + hypoxia + hypotens + ctclass + tsah, lrm, xtrans = rms.imp, data = TBI1)

# Runing the same regression above but just with the first complete dataset
lrm(unfav ~ trial + age + motor + pupil + hypoxia + hypotens + ctclass + tsah, data = TBI1.c1)

# Runnign the very same regression with a dataset with missing valeus
lrm(unfav ~ trial + age + motor + pupil + hypoxia + hypotens + ctclass + tsah, data = TBI1)


### Starting the example of imputations with the mice package
#### runing the imputation
### define the matrix pmat for mice   predictorMatrix  A value of '1' means that
### the column variable is used as a predictor for the target variable (in the
### rows). The diagonal of 'predictorMatrix' must be zero. In our matrix we dont
### want 'trial', 'age', 'motor', 'unfav' and 'mort' to be imputed.

#install.packages("mice")
# install.packages("mitml") # Algumas funes do mice dependenm desse pacote as no  listado como dependencia
library("mice")

# Remove last variable Age30
TBI1  <- TBI1[, -17]
names(TBI1)

# Create a matrix 16 x 16
p <- 16
pmat  <- matrix(rep(1,p*p),nrow=p,ncol=p)
diag(pmat)  <- rep(0,p)
pmat[,c(1:2, 10, 15:16)]  <- 0
pmat[ c(1:2, 10, 15:16),] <- 0
pmat
# These options are to chose which variables will be used in each pmm is not available at aregImpute

# Defines data to be used and the imputation method for each column, seed =1, usually, the default options are fine
mice.imp <- mice(TBI1, m = 10, predictorMatrix = pmat, seed=1, visitSequence = "roman")
mice.imp2 <- mice(TBI1, m = 10, seed=1, visitSequence = "revmonotone", method = "rf")

# Some diagnostics
fluxplot(TBI1) # ?fluxplot # for details on its interpretation

mice.imp2
# All the blues are original data, and all red are imputed data
# Mice 3.12.0 returns an error
bwplot(mice.imp2) # picks only the numeric variables, even the 0 1
densityplot(mice.imp2)
stripplot(mice.imp2)

# Create a completed dataset from mice imputations, second dataset
TBI1.c2 <- complete(mice.imp2, 2)
View(TBI1.c2)

## Adjusted analyses with all ten imputed dataset at the same time
fit.mice <- with(mice.imp, glm(unfav ~ trial + age + motor + pupil + hypoxia + hypotens + ctclass + tsah, family = binomial))

fit.mice2 <- with(mice.imp2, glm(unfav ~ trial + age + motor + pupil + hypoxia + hypotens + ctclass + tsah, family = binomial))
summary(fit.mice2)
summary(fit.mice2, "glance")
pool(fit.mice2)


fit.mice3 <- with(mice.imp2, glm(unfav ~ trial + age + pupil + hypoxia + hypotens + ctclass + tsah, family = binomial))
summary(fit.mice3)
summary(fit.mice3, "glance")
pool(fit.mice3)

# pool.compare(fit.mice2, fit.mice3, method = "likelihood")$pvalue
# pool.compare(fit.mice2, fit.mice3, method = "wald")$pvalue

D1(fit.mice2, fit.mice3)
D2(fit.mice2, fit.mice3)
D2(fit.mice2, fit.mice3, use = "likelihood")
D3(fit.mice2, fit.mice3)

# or with glm model function

# Deprecated
# fit.mice4 <- glm.mids(unfav ~ trial + age + motor + pupil + hypoxia + hypotens + ctclass + tsah, data = mice.imp, family = binomial)

fit.mice4 <- with(mice.imp, glm(unfav ~ trial + age + motor + pupil + hypoxia + hypotens + ctclass + tsah, family = binomial))
summary(fit.mice4)
pool(fit.mice4)

# fit.mult.imput from rms package also work with mice objects
fit.mice5 <- fit.mult.impute(unfav ~ trial + age + motor + pupil + hypoxia + hypotens + ctclass + tsah, lrm, xtrans = mice.imp2, data = TBI1, x = T, y = T)
fit.mice5

x <- fastbw(fit.mice5)
x

fit.mice6 <- fit.mult.impute(unfav ~ age + motor + pupil + hypoxia + hypotens + ctclass + tsah, lrm, xtrans = mice.imp2, data = TBI1, x = T, y = T)
fit.mice6
summary(fit.mice6)
# validate(fit.mice5, B = 1000)
# val.prob(logit = predict(fit.mice6), y = fit.mice6$y)

# lrm or glm regression with the completed dataset
# rms package
lrm(unfav ~ age + motor + pupil + hypoxia + hypotens + ctclass + tsah, data = TBI1.c2)
fit.mice6

# stats package
glm(unfav ~  age + motor + pupil + hypoxia + hypotens + ctclass + tsah, data = TBI1.c2, family = binomial)

# Detaching all packages
while(search()[2] != "tools:rstudio") detach(2)

# Dont forget to save the workspace, as these imputed data will be used later
save.image("D:/Banco/curso_CPM/2020/data/TBI1.RData")


######################################
######################################

rm(list =ls())
graphics.off()

load("D:/Banco/curso_cpm/2020/data/Dados_para_avaliacao.RData")
# save.image("D:/Banco/curso_cpm/2020/data/Dados_para_avaliacao.RData");ls()

# 3. Carregue no R o banco de dados f com o seguinte comando load("Dados_para_avaliacao.RData"). Usando a funo naclus() e naplot() com o banco de dados f, diga quais as variveis possuem mais dados ausentes em ordem decrescente.
# Escolha uma opo:
# 1. rxcardio, Q6.E1, Q7.E1 # <-
# 2. Q7.E1, Q6.E1, rxcardio.
# 3. Q6.E1, Q7.E1, Q41B.A2
# 4. NIU, d, chagas, set

library(rms)
x <- naclus(f)
naplot(x)
naplot(x, which = 'na per var')

# 4. Carregue no R o banco de dados f com o seguinte comando load("Dados_para_avaliacao.RData"). Usando a funo naclus() e naplot() com o banco de dados f, qual o maior nmero de dados ausentes em uma nica linha e quantas linhas possuem esse mesmo padro (nessa ordem).
# Escolha uma opo:
# 1. 33, 8 <-
# 2. 8, 33
# 3. 473, 0
# 4. 161,1
# 5. 1, 161

naplot(x, 'na per obs')
