# Remove todos os objetos da rea de trabalho
rm(list = ls())
while(search()[2] != "tools:rstudio") detach(2)

# Instala pacotes a serem utilizados se no estiverem j intalados
# install.packages("mfp")
# install.packages("gam")
# install.packages("rms")
# install.packages("visreg")
# install.packages(epiDisplay)
# install.packages(ems)

# Carrega os pacotes de funes a serem utilizadas
library(rms)
library(ems)

#--------
#Example 1 - GUSTOS-I - Interactions - Mortality after 30 days of infart
#--------

# declara o diretrio padro e importa o banco de dados do exemplo
setwd('C:/Banco/curso_CPM/2019/data/')
gustos <- spss.get("sample4.sav")
# Visualiza a estrutura do banco
str(gustos[,c("DAY30","AGE","KILLIP","HIG","DIA","HYP","HRT","TTR","SEX")])
# Visualiza as primeiras seis linhas do banco
head(gustos)

# save datadistributions at options workspace as many rms functions damands it
dd <- datadist(gustos)
options(datadist = NULL)
options(datadist = "dd")
# Para fazer contrastes de variveis ordenadas
options(contrasts=c("contr.treatment", "contr.treatment"))

# sapply(gustos[,c("DAY30","AGE","KILLIP","HIG","DIA","HYP","HRT","TTR","SEX")],table)
# Faz uma tabela de todas as variveis em relao ao desfecho para explorar os dados
tableStack(c("AGE","KILLIP","HIG","DIA","HYP","HRT","TTR","SEX"), by = DAY30, dataFrame = gustos, vars.to.factor = c("KILLIP","HIG","DIA","HYP","HRT","TTR","SEX"), test = FALSE, var.labels = FALSE, total.column = TRUE, NArow = TRUE)

#Poderamos colocar labels em variaveis transforma-las em fatores.
# factor(gustos[,c("DAY30")],levels = c(0,1),labels = c("naomorreu","morreu"))

# Repetindo os exemplos de regresso linear para previso com regresso logstica
# Como pacote stats
modglm <- glm(DAY30 ~ AGE + HIG + HRT + TTR, data = gustos, family = binomial)
modglm
AGE = 62; HIG = 1; HRT = 1; TTR = 1
Y <- -10.9678 + 0.1057 * AGE + 0.7677 * HIG + 1.0497 * HRT + 0.7138 * TTR
Y
(2.718282 ^ Y / (2.718282 ^ Y + 1))
(exp(Y) / (exp(Y) + 1))
1/(1 + exp(-Y))
1/(1 + 2.718282 ^ -Y)
plogis(Y)

Y <- predict(modglm, type = "link",
        newdata = data.frame(AGE = 62, HIG = 1, HRT = 1, TTR = 1))
plogis(Y)
predict(modglm, type = "response",
        newdata = data.frame(AGE = 62, HIG = 1, HRT = 1, TTR = 1))

# Com o pacote rms
modlrm <- lrm(DAY30~AGE + HIG + HRT + TTR, data = gustos)
modlrm
Y <- predict(modlrm, type = "lp",
             newdata = data.frame(AGE = 62, HIG = 1, HRT = 1, TTR = 1))
Y
plogis(Y)
predict(modlrm, type = "fitted.ind",
        newdata = data.frame(AGE = 62, HIG = 1, HRT = 1, TTR = 1))

# Intervalo de confiana da previso
Y <- predict(modlrm, type = "lp", se.fit = TRUE,
        newdata = data.frame(AGE = 62, HIG = 1, HRT = 1, TTR = 1))
Y
Y$linear.predictors # O nome desse objeto pode mudar se for a partir do glm
Y$se.fit
# imprimindo bonitinho no console
sprintf("%1$.3f [%2$.3f-%3$.3f]", plogis(Y$linear.predictors), plogis(Y$linear.predictors - 1.96*Y$se.fit), plogis(Y$linear.predictors + 1.96*Y$se.fit))

#Quais opes temos para o KILLIP?
table(gustos$KILLIP)
gustos$KILLIP2 <- factor(gustos$KILLIP, levels = c(1,2,3,4), labels = c("1","2","3/4","3/4"))
gustos$KILLIP3 <- as.ordered(gustos$KILLIP2)
table(gustos$KILLIP)
table(gustos$KILLIP2)
table(gustos$KILLIP3)

K_cont <- lrm(DAY30 ~ KILLIP, data = gustos)
K_cont
anova(K_cont)

K_factor <- lrm(DAY30 ~ KILLIP2, data = gustos)
K_factor
anova(K_factor)

K_order <- lrm(DAY30 ~ KILLIP3, data = gustos)
K_order
anova(K_order)
#Vamos usar o contnuo..

gustos$AGE65 <- ifelse(gustos$AGE <= 65,0,1)
modAGE <- lrm(DAY30 ~ AGE, data = gustos)
modAGE65 <- lrm(DAY30 ~ AGE65, data = gustos)
modAGErcs <- lrm(DAY30 ~ rcs(AGE,3), data = gustos)
# erro de ajuste  sinal que no precisa ou no  possivel
modAGEpoly <- lrm(DAY30 ~ poly(AGE,2), data = gustos)
library(mfp)
modAGEpoly2 <- mfp(DAY30 ~ fp(AGE), family = "binomial", data = gustos)
modAGE
modAGE65
modAGErcs
modAGEpoly
anova(modAGE)
anova(modAGE65)
anova(modAGErcs)
anova(modAGEpoly, test = "Chisq")
anova(modAGEpoly2, test = "Chisq")

# Efeito linear da idade
plot(gustos$AGE, modAGE$linear.predictors, type = "l")
# Efieto categorico em 65
points(gustos$AGE, modAGE65$linear.predictors)
# Efeito alisado pelo spline cubica
rcspline.plot(gustos$AGE, gustos$DAY30, nk = 3, statloc = NULL, add = T, lty = 2, plotcl = F, showknots = F)
# Efeito alisado pelo polinomio 2
lines(gustos$AGE, modAGEpoly$linear.predictors, lty = 4, col = "red")

# Interpretando uma sada de regresso logistica em OR
dd <- datadist(gustos)
options(datadist = NULL) # opcional para remover datadist anterior
options(datadist = "dd")
summary(modlrm) # o summary do lrm retorna as OR
summary(modglm) # summary do glm retorna o print do lrm
plot(summary(modlrm))
epiDisplay::logistic.display(modglm) # display retorna o bruto e o ajustado
round(exp(coefficients(modlrm)[2:5]),2)

# Comeando a explorar ineraes e formas funcionais
# Main effects for 8 predictors
full <- lrm(DAY30~AGE+KILLIP+HIG+DIA+HYP+HRT+TTR+SEX,data=gustos)

# Interactions with age
fullint <- lrm(DAY30 ~ AGE *(KILLIP+HIG+DIA+HYP+HRT+TTR+SEX), data = gustos)

# Examine contribution to fit of interactions
anova(fullint)
fastbw(fullint)

# Select only interaction AGE * HRT
fullints <- lrm(DAY30~AGE*HRT+KILLIP+HIG+DIA+HYP+TTR+SEX, data = gustos, x=T, y=T )
fullints
anova(fullints)
fastbw(fullints)

fullints <- lrm(DAY30~AGE*HRT+KILLIP+HIG+DIA+HYP+TTR+SEX,data=gustos, x = T, y = T)
plot(Predict(fullints))
plot(Predict(fullints, "AGE", "HRT"))

# Mesma coisa de outro jeito com plot interaction effect:
fullints2 <- glm(DAY30~AGE*HRT+KILLIP+HIG+DIA+HYP+TTR+SEX,data=gustos,family = "binomial")
library(lattice)
library(visreg)
visreg(fullints2)
visreg(fullints2, "AGE", by = "HRT", overlay = TRUE, partial = TRUE)

#Fazendo interacao da idade com HRT, excluindo os com HRT abaixo dos 55 anos
# No  possivel reproduzir o grafico do livro a no ser usando o mesmo moelo fullint e truncando os valores de idade para HRT = 1
znh <- data.frame(DIA = 0,
                  HYP = 0,
                  TTR = 1,
                  SEX = 0,
                  KILLIP = 1,
                  HIG = 0,
                  AGE = 20:80,
                  HRT = 0)
plot(znh$AGE, predict(fullints, znh), type = "l", ylim = c(-7,0), xlab = "Idade", ylab = "Preditor linear", lwd = 3)

zh55 <- data.frame(DIA = 0,
                   HYP = 0,
                   TTR = 1,
                   SEX = 0,
                   KILLIP = 1,
                   HIG = 0,
                   AGE = 56:80,
                   HRT = 1)
lines(zh55$AGE, predict(fullints2, zh55), lwd = 3, lty = 2)

# Variavel indicadora de idade > 55 e taquicardia presente
gustos$AGEH55 <- ifelse(gustos$AGE > 55 , gustos$AGE, 0)
# Variavel indicadora da idade quando a taquicardia ausente
gustos$AGEnH <- with(gustos, (1-HRT) * AGE)

# Variavel indicadora de interao e taquicardia com idade > 55
# gustos$AGE55I  <- ifelse(gustos$AGE > 55, 1, 0)
# head(gustos[,Cs(AGEnH,AGE)])

# Modelo especificado em que um efeito para ausencia de taquicardia e um efeito da taquicardia interagindo com idade interagindo com idade somente se maior que 55
fullints3 <- lrm(DAY30 ~ AGEnH + AGEH55 + KILLIP+HIG+DIA+HYP+TTR+SEX,data = gustos, x = T, y = T)
fullints3
fastbw(fullints3)

dd <- datadist(gustos)
options(datadist = NULL)
options(datadist = "dd")

plot(Predict(fullints3))
plot(Predict(fullints3, "AGEH55"))
plot(Predict(fullints3, "AGEnH"))
visreg(fullints3, "AGEnH")
visreg(fullints3, "AGEH55")

#--------
#Example 2 - Testicular cancer - Non-linearity
#--------
t821  <- spss.get('t821.sav')
names(t821) <- tolower(names(t821))

dd <- datadist(t821)
options(datadist = NULL)
options(datadist = "dd")

n544  <- t821[t821$study=="development",] # development set, n=544, JCO 1995
head(n544)

lrm(nec ~ ter + preafp + prehcg + lnldhst + sqpost + reduc10, data=n544)

# RCS: multivariable logistic regression with 3 rcs functions,
# each 4 knots
#library(rms)

rcspline.plot(n544$lnldhst, n544$nec, show = "prob", nk = 4, statloc = F, noprint = T, m = nrow(n544)/10)

rcspline.plot(n544$sqpost, n544$nec, show = "prob", nk = 4, statloc = F, noprint = T, m = round(nrow(n544)/10,0))

rcspline.plot(n544$reduc10, n544$nec, show = "prob", nk = 4, statloc = F, noprint = T, m = round(nrow(n544)/10,0))

# Explorando uma truncagem na var continua n544$sqpost
m_sqpost <- lrm(nec ~ sqpost, data=n544)
m_sqpost
m_sqpost_rcs <- lrm(nec ~ rcs(sqpost, 4), data=n544)
m_sqpost_rcs
plot(Predict(m_sqpost_rcs))
range(n544$sqpost)
n544$sqpost_trunc <- ems::trunc_num(n544$sqpost, min = 1.414214, max = 7.5)
range(n544$sqpost_trunc)
m_sqpost_trunc <- lrm(nec ~ sqpost_trunc, data=n544, y = T, x = T)
m_sqpost_rcs
m_sqpost_trunc
anova(m_sqpost_rcs)
anova(m_sqpost_trunc)
AIC(m_sqpost_rcs); AIC(m_sqpost_trunc)

# Ajustes modelos cheio com spline cubica restrita
m1 <- lrm(nec ~ ter + preafp + prehcg + rcs(lnldhst, 4) + rcs(sqpost, 4) + rcs(reduc10, 4), data=n544)
m1
plot(Predict(m1, "reduc10"))
plot(Predict(m1, "sqpost"))
plot(Predict(m1, "lnldhst"))
anova(m1)

# ordinary (non-orthogonal) polynomial
m12 <- lrm(nec ~ ter + preafp + prehcg + pol(lnldhst, 2)+ pol(sqpost, 2) + pol(reduc10, 2), data=n544)
m12
# attr(terms(m12),"predvars")
plot(Predict(m1))
plot(Predict(m12))
plot(Predict(m12, "reduc10"))
plot(Predict(m1, "reduc10"))
plot(Predict(m12, "sqpost"))
plot(Predict(m1, "sqpost"))
plot(Predict(m12, "lnldhst"))
plot(Predict(m1, "lnldhst"))
anova(m12)
AIC(m1)
AIC(m12)

# FP: multivariable fractional polynomial
library(mfp)
m2 <- mfp(nec ~ ter + prehcg + fp(lnldhst) + fp(sqpost) + fp(reduc10) , family = "binomial", data = n544)
# m2 <- mfp(nec ~ ter + preafp + prehcg + fp(lnldhst) + fp(sqpost) + fp(reduc10), family = "binomial", data = n544) # something went wrong with "preafp"
m2
summary(m2)
val.prob(y = n544$nec, logit = m2$fitted.values, pl = FALSE)
quiq <- m2$null.deviance-m2$deviance; quiq
coefficients(m2)
visreg(m2, "lnldhst")
visreg(m2, "sqpost")
visreg(m2, "reduc10")

# Funes de transformes que poderiam ser utilizadas dentro do glm
# s  de spline e df  uma referencia aos numero de coeficientes
# lo  de loess
# bs  polynomial splines
# poly  Orthogonal Polynomials = pol no rms
# ns  natural cubic splines

# GAM: multivariable gam, 3 effective df for each continuous predictor
library(gam)
m3 <- gam(nec ~ ter + preafp + prehcg + s(lnldhst,3) + s(sqpost,3) + s(reduc10,3), data=n544, family = binomial, y = T, x = T)
m3
quiq <- m3$null.deviance-m3$deviance; quiq
terms(m3)
summary(m3)
val.prob(y = n544$nec, logit = m3$fitted.values, pl = T)
# visreg(m3, xvar = "lnldhst")

# multivariable gam, optimal effective df for each continuous predictor
# based on generalized cross-validation (GCV)
library(mgcv)
m4 <- gam(nec ~ ter + preafp + prehcg + s(lnldhst) + s(sqpost) + s(reduc10), data= n544, family = binomial)
m4
m0 <- gam (nec~1, data=n544, family=binomial)
quiq <- m4$null.deviance-m4$deviance; quiq
summary(m4)
visreg(m4, "lnldhst")
visreg(m4, "sqpost")
visreg(m4, "reduc10")

# Selcionando por anlise univariada com limite de .10 no teste Wald
# Primeiro passo
lrm(formula = DAY30 ~ AGE, data = gustos, x = T, y = T) # Sim
lrm(formula = DAY30 ~ HRT, data = gustos, x = T, y = T) # Sim
lrm(formula = DAY30 ~ KILLIP, data = gustos, x = T, y = T) # Sim
lrm(formula = DAY30 ~ HIG, data = gustos, x = T, y = T) # Sim
lrm(formula = DAY30 ~ DIA, data = gustos, x = T, y = T) # No
lrm(formula = DAY30 ~ HYP, data = gustos, x = T, y = T) # No
lrm(formula = DAY30 ~ TTR, data = gustos, x = T, y = T) # Sim
lrm(formula = DAY30 ~ SEX, data = gustos, x = T, y = T) # No

# Segundo passo ajustar os inicialmente significantes
lrm(formula = DAY30 ~ AGE + HRT + KILLIP + HIG + TTR, data = gustos, x = T, y = T)
# Todos retidos porque todos p de Wald so menores que .05. Repare que alguns efeitos e p so diferentes do univariados

# selecinando preditores pelo AIC pelo glm e pelo lrm
fullints_glm <- glm(formula = DAY30 ~ AGE * HRT + KILLIP + HIG + DIA + HYP +
      TTR + SEX, data = gustos, x = T, y = T, family = binomial)
step(fullints_glm, direction = "backward")
anova(fullints_glm, test = "Chisq")

# a funo equivalente ao step no rms  a fastbw
# o argumento force pode ser utilizado para no tentar remover uma ou mais var
fullints
fastbw(fullints)
fastbw(fullints, type = "individual")
fastbw(fullints, rule = "p")

# selecinando preditores pelo AIC e bootstrap
fullints
fastbw(fullints)
set.seed(1)
x <- validate(fullints, bw = T, B = 5000) # Leva uns minutos pra rodar
print(x)
attributes(x)
xx <- sapply(1:ncol(attr(x, "kept")),
       function(i) prop.table(table(attr(x, "kept")[,i])))
colnames(xx) <- colnames(attr(x, "kept"))
xx

#### Exercicio
# Carregue a base de dados: "Dados_para_avaliacao.RData" utilizando a funo load("Dados_para_avaliacao.RData"). Aplique o mtodo "Anlise univariada e especificao do modelo" para os preditores: Q5.A2, Q6.A2, Q17.A2, Q18.A2 e Q19.A2 ("Sex", "Age", "Ever lived in rural rea", "Ever lived in mug houses", "recognize the kissing bug", respectivamente), no desfecho: d ("chagas diagnosis"), da base de dados f, utilizando um modelo logstico. Na primeira etapa do mtodo, utilize o critrio p<0.15 e na segunda etapa utilize o critrio p<0.05. Quais foram os preditores selecionados para o modelo final que analisa a chance de ter doenas de chagas?
rm(list = ls())
graphics.off()
load("Dados_para_avaliacao.RData");ls()

library(rms)
dd <- datadist(f)
options(datadist = "dd")

lrm(d ~ Q5.A2, data = f) # No
lrm(d ~ Q6.A2, data = f) # Sim
lrm(d ~ Q17.A2, data = f) # Sim
lrm(d ~ Q18.A2, data = f) # Sim
lrm(d ~ Q19.A2, data = f) # Sim
lrm(d ~ Q6.A2 + Q17.A2 + Q18.A2 + Q19.A2, data = f) # Sai Q6 e Q18
lrm(d ~ Q17.A2 + Q19.A2, data = f) # Final

# Questo
fullmod <- lrm(d ~ Q5.A2 + Q6.A2 + Q17.A2 + Q18.A2 + Q19.A2, data = f, x = T, y = T)
fastbw(fullmod)
fastbw(fullmod, type = "individual")

fullmod <- glm(d ~ Q5.A2 + Q6.A2 + Q17.A2 + Q18.A2 + Q19.A2, data = f, x = T, y = T, family = binomial)
step(fullmod)


# Questa
idade_lin <- lrm(d ~ Q6.A2, data = f, x = T, y = T)
idade_rcs <- lrm(d ~ rcs(Q6.A2,3), data = f, x = T, y = T)
idade_fp <- lrm(d ~ pol(Q6.A2), data = f, x = T, y = T)
quantile(f$Q6.A2, na.rm = T)
idade_s <- lrm(d ~ lsp(Q6.A2, c(38,49,60)), data = f, x = T, y = T)

anova(idade_lin) # 12.55
anova(idade_rcs) # 12.55
anova(idade_fp)
anova(idade_s)
AIC(idade_lin); AIC(idade_rcs); AIC(idade_fp); AIC(idade_s)

plot(Predict(idade_lin))
plot(Predict(idade_rcs))
plot(Predict(idade_fp))
plot(Predict(idade_s))

# library(visreg)
# visreg(idade_fp)

f$idade49 <- ifelse(f$Q6.A2 < 50, f$Q6.A2, 50)
f$idade50 <- ifelse(f$Q6.A2 >= 50, f$Q6.A2, 50)

idade_49 <- lrm(d ~ idade49 , data = f, x = T, y = T)
idade_50 <- lrm(d ~ idade50 , data = f, x = T, y = T)
idade_4950 <- lrm(d ~ idade50 + idade49, data = f, x = T, y = T)
anova(idade_4950)
anova(idade_49)
anova(idade_50)
dd <- datadist(f)
options(datadist = "dd")
plot(Predict(idade_4950))

anova(idade_fp);anova(idade_4950)
AIC(idade_fp);AIC(idade_4950)

# Questo
full_idade_fp <- lrm(d ~ Q5.A2 + pol(Q6.A2) + Q17.A2 + Q18.A2 + Q19.A2, data = f, x = T, y = T)
rms::vif(full_idade_fp)
