#------------------
#Example 1 - Testicular cancer - Performance
#------------------
# install.packages("rms")

rm(list = ls())
graphics.off()

setwd('C:/Banco/curso_CPM/2019/data/')

library(rms)

t821  <- spss.get('t821.sav')
dim(t821)

# Fracionando a banco para desenvolvimento do modelo
# development set, n=544, JCO 1995
# Essa coluna possui uma de desenvolvimento e outras 3 possiveis para validao
n544  <- t821[t821$STUDY == "development",]
dim(n544)

# 5 predictors in data set n544; develop model
full <- lrm(NEC ~ TER+PREAFP+PREHCG+SQPOST+REDUC10, data = n544, x = TRUE, y = TRUE)
full

#_____________
# apparent
#_____________
# Estima medidas de de discriminao e de calibrao
ap <- rms::val.prob(logit = full$linear.predictor, y = full$y, g = 10)
round(ap, 3)

##############################
# O ranque de correo de Somers Dxy  uma medida que representa a diferena entre a probabilidade de um par do desfecho observado e probabilidade prevista pelo modelo serem concordantes e a probabilidade de um par de desfecho e previso serem discordantes. Esse par  dito concordante se valores maiores de previso so pareados com a presena do desfecho, e discordante se valores maiores de previso so pareados com a ausncia do desfecho. Esse ndice varia de -1 a 1. O 0 indica que no h correo. Valores maiores que 0 indicam que o modelo possui valor discriminativo. Valores menores que 0 podem existir mas no fazem sentido no contexto de modelos preditivos.

# A rea sob a curva (AUC) ROC  interpretada como a probabilidade de que o classificador ir atribuir valores maiores para um sujeito com o desfecho escolhido aleatoriamente do que um sujeito sem o desfecho escolhido aleatoriamente. Esta medida tambm poderia ser interpretada como uma sensibilidade mdia para uma variao de especificidade para todos os possveis valores do classificador (modelo) ou vice versa. A AUC ROC varia entre 0.5 e 1, e quanto mais prximo de 1 melhor a discriminao.

# R2  uma medida que verifica o quanto que se as previses se afastam ou se aproximam dos valores observados. Usualmente essa medida  interpretada como a frao da variao total do desfecho que o modelo consegue explicar. Essa medida varia entre 0 e 1 quanto mais prximo de 1 melhor o modelo explica a variao do desfecho.

# O ndice de discriminao D  um ndice que expressa a diferena entre a quantidade de desfechos no grupo com previses altas e a quantidade de desfechos no grupo com previses baixas. Essa diferena  ponderada pelo tamanho do grupo. Esse ndice varia entre +1 e -1. Valores maiores que 0.4 so indicativos de boa discriminao, e valores menores que 0.2 indicam uma discriminao ruim.

# \textit{Unreliability index U}  um ndice de calibrao, ou seja, indica o quanto as previses se afastam dos valores observados. Se h uma coincidncia perfeita entre o observado e o previsto, o grfico de calibrao indicar uma reta de 45 (slope = 1), e o valor zero da previso ir coincidir com o valor zero observado (intercepto = 0). Esse ndice testa (hiptese nula) simultaneamente se o slope  igual a 1 e o intercepto  igual 0. Esse ndice possui, como todo teste estatstico, um valor p correspondente. Valores de p muito baixos indicam um modelo mal calibrado.

# O \textit{quality index Q}  mais uma medida de discriminao. Esse ndice varia entre 0 e 1. Quanto mais prximo de 1 melhor o poder discriminativo do modelo.

# O Escore de Brier  uma medida de discriminao que representa o quadrado das diferenas entre o observado e o previsto pelo modelo. Esse ndice varia entre 0 e 0.5 se a prevalncia do desfecho for 0.5. Caso contrrio, amplitude de possveis resultados pode variar, por exemplo: para prevalncia de desfecho de 0.10 o mximo do escore de Brier seria 0.09. Ao contrario de outros ndices, valores mais prximos de 0 indicam melhor discriminao.

# O Spiegelhalter Z-test para calibrao, possui como hiptese nula que a probabilidade observada  igual a probabilidade prevista para todos os valores das probabilidades observadas. Assim como todos os testes estatsticos possui um valor p.  p valores pequenos indicam que o modelo  mal calibrado.

# O \textit{calibration belt} cria bandas das previses no grfico que compara as previses com as probabilidades observadas.\cite{nattino_new_2014} Se a correspondncia for perfeita a bissetriz do grfico estar dentro da banda em toda a amplitude. Esse grfico tambm indica qual amplitude de probabilidades observadas no est includa dentro da banda. Essa anlise tambm possui um teste correspondente. Valores de p muito baixos indicam que o modelo  mal calibrado.

##############################
# Funo com medidas de validade (scaled Brier como adicional)
source("C:/Banco/curso_CPM/2019/scripts/val.prob.ci.June09.r")
ap2 <- val.prob.ci(logit = full$linear.predictors, y = full$y, g = 15)
round(ap2,3)

# Estimando as probabilidades aplicando a funo logistica ao Y da equao
# prob <- exp(full$linear.predictors)/(1+exp(full$linear.predictors))
prob <- plogis(full$linear.predictors)

# Calculando o Calibration belt
# install.packages("givitiR")
library(givitiR)
cb <- givitiCalibrationBelt(full$y, prob, "internal")
plot(cb)

# Calculando o discrimination slope
DL <- mean(prob[which(full$y == 1)]) - mean(prob[which(full$y == 0)])
round(DL, 3)
# [1] 0.301
boxplot(list(prob[which(full$y == 1)], prob[which(full$y == 0)]))
stripchart(list(prob[which(full$y == 1)], prob[which(full$y == 0)]), vertical =T, method = "jitter")
x0 <- c(.8,1.8)
x1 <- c(1.2, 2.2)
y1 <- y0 <- c(mean(prob[which(full$y == 1)]), mean(prob[which(full$y == 0)]))
segments(x0 = x0, x1 = x1, y0 = y0, y1 = y1, lwd = 4)
rm(x0, x1, y0, y1)

# Estatistica de calibrao de Hosmer-Lemeshow
source("C:/Banco/curso_CPM/2019/scripts/HLtest.r")
hl.ext2(prob,full$y)
# p-value 0.7468576

# Estatistica de calibrao de Hosmer-Lemeshow com o pacote PredictABEL
# install.packages("PredictABEL")
library(PredictABEL)
plotCalibration(data = n544, cOutcome = "NEC", predRisk = prob)

library(ems)
calcurve(deaths  = n544$NEC, pred = prob * 100, table = F, y2axis_label = "Predicted outcome (%)") # um subtitulo indesejavel que no sai.

#_____________
# Internal validation with 200 bootstraps
#_____________

set.seed(1)
int <- validate(full, B=200);
int
int <- validate(full, B=1000);
int

############## Penalizando o modelo com o Bootstrap
# copy original model fit to shrunk model
full.pen <- full
full.pen$coefficients

# use result from bootstrapping to shrink coefficients
full.pen$coefficients <- int["Slope","index.corrected"] * full$coefficients
full.pen$coefficients

# Estimate new intercept, with shrunk lp as offset variable, i.e. coef fixed at unity
full.pen$coefficients[1] <- lrm.fit(y = full.pen$y, offset = full.pen$x %*% full.pen$coefficients[1:5])$coefficients[1]

# MOdelo penalizado... so os coeficientes servem
full.pen
# Passadno os preditores lineares para o objeto dentro do modelo # opcional
full.pen$linear.predictors <- predict(full.pen, n544)

# Repetindo a avaliao de desempenho nos dados de desemvolvimento com o modelo penalizado

ap <- val.prob(logit = predict(full.pen, n544), y = full.pen$y, g = 15)
round(ap,3)

# Funo com medidas de validade (scaled Brier como adicional)
source("C:/Banco/curso_CPM/2019/scripts/val.prob.ci.June09.r")
ap2 <- val.prob.ci(logit = predict(full.pen, n544), y = full.pen$y, g = 15)
round(ap2,3)

# Estimando as probabilidades aplicando a funo logistica ao Y da equao
# prob <- exp(full$linear.predictors)/(1+exp(full$linear.predictors))
prob <- predict(full.pen, n544, type = "fitted.ind")

# Calculando o Calibration belt
# install.packages("givitiR")
library(givitiR)
cb <- givitiCalibrationBelt(full.pen$y, prob, "internal")
plot(cb)

# Calculando o discrimination slope
DL <- mean(prob[which(full.pen$y == 1)]) - mean(prob[which(full.pen$y == 0)])
round(DL, 3)
# [1] 0.301
boxplot(list(prob[which(full$y == 1)], prob[which(full$y == 0)]))
stripchart(list(prob[which(full$y == 1)], prob[which(full$y == 0)]), vertical =T, method = "jitter")
x0 <- c(.8,1.8)
x1 <- c(1.2, 2.2)
y1 <- y0 <- c(mean(prob[which(full$y == 1)]), mean(prob[which(full$y == 0)]))
segments(x0 = x0, x1 = x1, y0 = y0, y1 = y1, lwd = 4)
rm(x0, x1, y0, y1)

# Estatistica de calibrao de Hosmer-Lemeshow
source("C:/Banco/curso_CPM/2019/scripts/HLtest.r")
hl.ext2(prob,full.pen$y)
# p-value 0.7468576

# Estatistica de calibrao de Hosmer-Lemeshow com o pacote PredictABEL
# install.packages("PredictABEL")
library(PredictABEL)
plotCalibration(data = n544, cOutcome = "NEC", predRisk = prob)

# Repetindo a avaliao de desempenho com o modelo penalizado agora com em dados independentes
# Separando os dados de validao
val  <- t821[t821$STUDY == "indiana",]

# Estimando as probabilidades aplicando a funo logistica ao Y da equao
# prob <- exp(full$linear.predictors)/(1+exp(full$linear.predictors))
prob <- predict(full.pen, val, type = "fitted.ind")

ap <- val.prob(p = prob, y = val$NEC, g = 15)
round(ap,3)

# Funo com medidas de validade (scaled Brier como adicional)
source("C:/Banco/curso_CPM/2019/scripts/val.prob.ci.June09.r")
ap2 <- val.prob.ci(p = prob, y = val$NEC, g = 15)
round(ap2,3)

# Calculando o Calibration belt
# install.packages("givitiR")
library(givitiR)
cb <- givitiCalibrationBelt(val$NEC, prob, "external")
plot(cb)

# Calculando o discrimination slope
DL <- mean(prob[which(val$NEC == 1)]) - mean(prob[which(val$NEC == 0)])
round(DL, 3)
# [1] 0.301
boxplot(list(prob[which(val$NEC == 1)], prob[which(val$NEC == 0)]))
stripchart(list(prob[which(val$NEC == 1)], prob[which(val$NEC == 0)]), vertical =T, method = "jitter")
x0 <- c(.8,1.8)
x1 <- c(1.2, 2.2)
y1 <- y0 <- c(mean(prob[which(val$NEC == 1)]), mean(prob[which(val$NEC == 0)]))
segments(x0 = x0, x1 = x1, y0 = y0, y1 = y1, lwd = 4)
rm(x0, x1, y0, y1)

# Estatistica de calibrao de Hosmer-Lemeshow
source("C:/Banco/curso_CPM/2019/scripts/HLtest.r")
hl.ext2(prob,val$NEC)
# p-value 0.7468576

# Estatistica de calibrao de Hosmer-Lemeshow com o pacote PredictABEL
# install.packages("PredictABEL")
library(PredictABEL)
plotCalibration(data = val, cOutcome = "NEC", predRisk = prob)

##########################
##########################
########################## Exercicio
rm(list = ls())
graphics.off()

load("Dados_para_avaliacao.RData");ls()

library(rms)
# Questo 11 e 12
fit11 <- lrm(chagas ~ Q5.A2 + Q6.A2 + Q17.A2 + Q18.A2 + Q19.A2 + Q20.A2 + Q25.A2 + rbbb + EV, data = subset(f, subset = f$set == "Development"), x = T, y = T) #
fit11
fastbw(fit11) # Q17.A2 Q18.A2 Q19.A2 Q20.A2 Q25.A2 rbbb   EV

# Questo 13
fit13 <- lrm(chagas ~ Q17.A2 + Q18.A2 + Q19.A2 + Q20.A2 + Q25.A2 + rbbb + EV, data = subset(f, subset = f$set == "Development"), x = T, y = T) #
set.seed(1)
val_fit13 <- validate(fit13, B = 1000)
val_fit13

des <- val.prob(logit = fit13$linear.predictors, y = fit13$y, g = 10)

val <- val.prob(logit = predict(fit13, subset(f, subset = f$set == "Validation")),
         y = f[f$set == "Validation", "chagas"], g = 10)
round(des,3)
round(val,3)

# a Verdade
# b Verdade
# c Falso
# d verdadeiro


x <- c(1:10)
set.seed(1)
sample(x, 3)
